function [ w,of,time ] = SGD_all( x, y,option )
%x is nSample by nDimension 
%y is nSample by 1
%option.eta is the initial eta value
%option.model: SGD baseline
%NSGD, naive SGD has the same learning rate as SGD. can gurantee perform
%consistently better than SGD with the same learning rate
%CSGD, 2 phase learning rate CSGD, can tolerate larger learning rate

%output
%w: the final coefficient
%of: the objective function value - the optimal objective function value
%time: the time for each step
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Yang Mu
%yangmu@cs.umb.edu
%UMass Boston 
%2013 Feb.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n, d] = size(x);
mx = 0;
my = 0;
w = zeros(d,1);
v = w;
of = zeros(n*option.Maxiter,1);
time = zeros(n*option.Maxiter,1);
wstar = pinv(x'*x)*x'*y;
error = norm(y-x*wstar);
eta1 = option.eta;
c = 1/2;
t = 1;
visit = 0;
tic
for iter = 1:option.Maxiter
    for i = 1:n
        g_t = -x(i,:)'*(y(i)-x(i,:)*w);
        switch option.model
            case 'SGD'
                eta = eta1*t^(-c);
                w = w-eta*g_t;
                v = w;
            case 'NSGD'
                eta = eta1*t^(-c);
                if t <= n
                    mx = (t-1)/t*mx+1/t*x(i,:);
                    my = (t-1)/t*my+1/t*y(i);
                end
                %                                 P = eye(d)-mx'*mx/(mx*mx');
                %                                 w= P*((w-eta*g_t))+my/(mx*mx')*mx';
                tmpw = w-eta*g_t;
                w = tmpw-mx'*(mx*tmpw)/(mx*mx')+my/(mx*mx')*mx';
                v = w;
            case 'CSGD'
                %2 Phase leaning rate
                eta = eta1*t^(-c);
                if t <= n
                    mx = (t-1)/t*mx+1/t*x(i,:);
                    my = (t-1)/t*my+1/t*y(i);
                end
                tmpw = w-eta*g_t;
                if norm(mx) < 1e-10
                    mx = zeros(1,d);
                end
                 tmpr = my/(mx*mx'+eps)*mx';
                if t>=0.5*n && visit == 0 %0.5 is a parameter
                    c = 1;
                    eta1 = 1.5*eta1*sqrt(t); %1.5 is a parameter
                    visit = 1;
                end
                w = tmpw-mx'*(mx*tmpw)/(mx*mx'+eps)+tmpr;
                v = w;
        end
        of(t)=norm(y-x*v)-error;
        time(t) = toc;
        t = t+1;
    end
end
w = v;

